package OO::Attributes;
# $Id: Attributes.pm,v 1.3 2004/09/06 13:12:04 tina Exp $#
use 5.006001;
use strict;
use warnings;

use base 'Exporter';

our $VERSION = '0.01';


sub import {
	my $self = shift;
	return unless @_;
	my ($sub, $names) = @_;
	my $class = caller();
	no strict 'refs';
	# set the general method
	*{$class . '::_attribute_ref' } = $sub;
	for my $name (@$names) {
		my $string = $class . '::' . $name;
		*{$string} = sub {
			use strict;
			my $self = shift;
			# set method for each attribute
			return ${ $self->_attribute_ref($name) } unless @_;
			${ $self->_attribute_ref($name) } = shift;
		};
	}
}

1;
__END__

=head1 NAME

OO::Attributes - Perl extension for generic object-attributes

=head1 SYNOPSIS

  package Parent;
  use OO::Attributes sub {
    my $self = shift;
    my $name = shift;
    # we have an ordinary hashref for the object
    return \$self->{"_$name"};
  }, [qw(city name)];
  package Child;
  use base "Parent";
  use OO::Attributes sub {
    my $self = shift;
    my $name = shift;
    # object is arrayref, and the attributes are stored in element 0
    return \$self->[0]->{"_$name"};
  };
  package main;
  my $parent = {};
  bless($parent, "Parent");
  $parent->name("parent");
  my $child  = [];
  bless ($child, "Child");
  $child->name("child");
  # $parent->{_name} eq "parent";
  # $child->[0]->{_name} eq "child";


=head1 DESCRIPTION

This module provides a way to quickly set up attribute methods for objects.
Usually one just stores an attribute "age" in C<$self-E<gt>{_age}> or something
like that. If you want to write inhertiable modules you would just define
that once and access the attribute through a method age(), so that you
just have to change that method if you change your implementation from a hash
to an array, for example. The above example Shows this. Parent stores
its attributes in e.g. C<$self-E<gt>{_name}>, Child stores it in
C<$self-E<gt>[0]-E<gt>{_name}>. In the parent packagea you say

  use OO::Attributes sub {
    # how to access the attribute
    my $self = shift;
    my $name = shift;
  }, [qw(city name)];

The list of method names is passed as the arraref C<[qw(city name)]>.
In the Child package you don't need to pass that list if Child inherits
these methods.

=head2 EXPORT

None.

=head1 SEE ALSO

perlboot, perltoot, Class::MethodMaker

=head1 AUTHOR

tina, tinita at cpan . org

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2004 by tina

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.3 or,
at your option, any later version of Perl 5 you may have available.


=cut
